/*
  Scan QL keyboard matrix & send corresponding characters
  over USB.
  
  This stuff written by Jeroen Janmaat, 2012.
  Used vusb-for-arduino library v005 and Arduino IDE v1.0.1
  2012/06/14 First version
*/

#include "UsbKeyboard.h"

// If the timer isr is corrected
// to not take so long change this to 0.
#define BYPASS_TIMER_ISR 1

  int ii;
  int jj;
  int tt = 0; // time counter
  int effect = 0; // read line
  int rows[9] = {3, 6, 7, 8, 9, 10, 11, 12, A5}; // Pins for row scanning
  int QLrow = 0;
  int QLcol = 0;
  int QLrow_old = 0;
  int QLcol_old = 0;
  int sentonce = 0;
  byte keys[11][9] = {
    {NO_KEY, NO_KEY, NO_KEY,    NO_KEY, NO_KEY,        NO_KEY,     NO_KEY,    NO_KEY,          KEY_CTRL},
    {NO_KEY, NO_KEY, NO_KEY,    NO_KEY, NO_KEY,        NO_KEY,     NO_KEY,    NO_KEY,          KEY_SHIFT},
    {KEY_F1, KEY_2,  KEY_W,     KEY_3,  MOD_CAPS_LOCK, KEY_Z,      NO_KEY,    KEY_ARROW_LEFT,  NO_KEY},
    {KEY_F2, KEY_Q,  KEY_TAB,   KEY_1,  KEY_S,         KEY_C,      KEY_X,     KEY_ESCAPE,      NO_KEY},
    {KEY_F3, KEY_E,  KEY_R,     KEY_A,  KEY_F,         KEY_B,      KEY_V,     KEY_ARROW_RIGHT, NO_KEY},
    {KEY_4,  KEY_T,  KEY_Y,     KEY_D,  KEY_G,         KEY_M,      KEY_N,     KEY_SPACE,       NO_KEY},
    {KEY_5,  KEY_6,  KEY_I,     KEY_H,  KEY_K,         KEY_PERIOD, NO_KEY,    KEY_ARROW_UP,    NO_KEY},
    {KEY_7,  KEY_U,  KEY_O,     KEY_J,  KEY_SEMICOLON, KEY_QUOTE,  KEY_COMMA, KEY_ARROW_DOWN,  NO_KEY},
    {KEY_F4, KEY_8,  KEY_9,     KEY_L,  KEY_L_BRKT,    KEY_R_BRKT, NO_KEY,    KEY_ENTER,       NO_KEY},
    {KEY_F5, KEY_0,  KEY_MINUS, KEY_P,  KEY_EQUALS,    KEY_TILDE,  KEY_SLASH, KEY_BSLASH,      NO_KEY},
    {NO_KEY, NO_KEY, NO_KEY,    NO_KEY, NO_KEY,        NO_KEY,     NO_KEY,    NO_KEY,          KEY_ALT} };
  byte capsstate = 0;
  byte ctrlkey = 0;
  byte shiftkey = 0;
  byte altkey = 0;
  byte key_to_send = 0;

void setup() {                
  // configure the analog inputs as a digital output.
  pinMode(A0, OUTPUT); // set pin to output     
  pinMode(A1, OUTPUT);     
  pinMode(A2, OUTPUT);     
  pinMode(A3, OUTPUT);     
  // configure digital outputs.
  pinMode(3,  INPUT); // set pin to input
  pinMode(6,  INPUT);   
  pinMode(7,  INPUT);     
  pinMode(8,  INPUT);     
  pinMode(9,  INPUT);     
  pinMode(10, INPUT);     
  pinMode(11, INPUT);
  pinMode(12, INPUT);     
  pinMode(A5, INPUT); // pin 13 gives problems here
  digitalWrite(3,  HIGH); // turn on pullup resistors  
  digitalWrite(6,  HIGH);
  digitalWrite(7,  HIGH);
  digitalWrite(8,  HIGH);
  digitalWrite(9,  HIGH);
  digitalWrite(10, HIGH);
  digitalWrite(11, HIGH);
  digitalWrite(12, HIGH);
  digitalWrite(A5, HIGH);
  pinMode(13, OUTPUT); // Caps lock LED   
  Serial.begin(115200);

  #if BYPASS_TIMER_ISR
    // disable timer 0 overflow interrupt (used for millis)
    TIMSK0&=!(1<<TOIE0); // ++
  #endif

}

#if BYPASS_TIMER_ISR
void delayMs(unsigned int ms) {
   /*
  */ 
  for (int i = 0; i < ms; i++) {
    delayMicroseconds(1000);
  }
}
#endif

void loop() {
  UsbKeyboard.update();
    
  for(ii = 1; ii < 12; ii++) {
    switch(ii){
      case 1:
        digitalWrite(A0,LOW);
        digitalWrite(A1,LOW);
        digitalWrite(A2,LOW);
        digitalWrite(A3,LOW);
        break;
      case 2:
        digitalWrite(A0,HIGH);
        digitalWrite(A1,LOW);
        digitalWrite(A2,LOW);
        digitalWrite(A3,LOW);
        break;
      case 3:
        digitalWrite(A0,LOW);
        digitalWrite(A1,HIGH);
        digitalWrite(A2,LOW);
        digitalWrite(A3,LOW);
        break;
      case 4:
        digitalWrite(A0,HIGH);
        digitalWrite(A1,HIGH);
        digitalWrite(A2,LOW);
        digitalWrite(A3,LOW);
        break;
      case 5:
        digitalWrite(A0,LOW);
        digitalWrite(A1,LOW);
        digitalWrite(A2,HIGH);
        digitalWrite(A3,LOW);
        break;
      case 6:
        digitalWrite(A0,HIGH);
        digitalWrite(A1,LOW);
        digitalWrite(A2,HIGH);
        digitalWrite(A3,LOW);
        break;
      case 7:
        digitalWrite(A0,LOW);
        digitalWrite(A1,HIGH);
        digitalWrite(A2,HIGH);
        digitalWrite(A3,LOW);
        break;
      case 8:
        digitalWrite(A0,HIGH);
        digitalWrite(A1,HIGH);
        digitalWrite(A2,HIGH);
        digitalWrite(A3,LOW);
        break;
      case 9:
        digitalWrite(A0,LOW);
        digitalWrite(A1,LOW);
        digitalWrite(A2,LOW);
        digitalWrite(A3,HIGH);
        break;
      case 10:
        digitalWrite(A0,HIGH);
        digitalWrite(A1,LOW);
        digitalWrite(A2,LOW);
        digitalWrite(A3,HIGH);
        break;
      case 11:
        digitalWrite(A0,LOW);
        digitalWrite(A1,HIGH);
        digitalWrite(A2,LOW);
        digitalWrite(A3,HIGH);
        break;
    }

    ///////////////////////////////////////
    // With one column selected, scan rows
    ///////////////////////////////////////
    for(jj = 0; jj < 9; jj++) {
      effect = (!digitalRead(rows[jj]));
      // Any key among the normal characters pressed?
      if((effect) && (ii>2) && (ii<11) && (jj<8)) {
        QLcol = ii;
        QLrow = jj;
      }
      if((ii==1) && (jj==8))
        ctrlkey = effect;
      if((ii==2) && (jj==8))
        shiftkey = effect;
      if((ii==11) && (jj==8))
        altkey = effect; 
    }
  }
  
  //////////////////////////////////
  // Special case for Caps Lock key
  //////////////////////////////////
  if((QLcol == QLcol_old)&&(QLrow==QLrow_old)&&(QLcol == 3) && (QLrow == 4)&&(sentonce!=keys[QLcol-1][QLrow])) {
    sentonce = keys[QLcol-1][QLrow];
    if(capsstate) {
      capsstate = 0;
      digitalWrite(13,LOW);
    }
    else {
      capsstate = 1; 
      digitalWrite(13,HIGH);
    }
  }
  
  ////////////////////////////////////
  // Action on key when first pressed
  ////////////////////////////////////
  if((QLcol == QLcol_old)&&(QLrow==QLrow_old)&&(QLcol!=0)&&(sentonce!=keys[QLcol-1][QLrow])) {
    //Serial.println(keys[QLcol-1][QLrow]);
    sentonce = keys[QLcol-1][QLrow];
    tt = 0;
    key_to_send = keys[QLcol-1][QLrow];
    // Exceptions; especially assigned key combinations (Backspace, delete, home, end, pgup, pgdown, F6-F10)
    if(((QLrow==7)&&((QLcol==3)||(QLcol==5)||(QLcol==7)||(QLcol==8))&&((altkey)||(ctrlkey)))||(((QLcol==3)||(QLcol==4)||(QLcol==5)||(QLcol==9)||(QLcol==10))&&(QLrow==0)&&(shiftkey))) {
      if((QLrow==7)&&(QLcol==3)&&(ctrlkey))   UsbKeyboard.sendKeyStroke(KEY_BACKSPACE);  
      if((QLrow==7)&&(QLcol==5)&&(ctrlkey)&&(!altkey))   UsbKeyboard.sendKeyStroke(KEY_DELETE);  
      if((QLrow==7)&&(QLcol==7)&&(ctrlkey))   UsbKeyboard.sendKeyStroke(KEY_PGUP);  
      if((QLrow==7)&&(QLcol==8)&&(ctrlkey))   UsbKeyboard.sendKeyStroke(KEY_PGDOWN);  
      if((QLrow==7)&&(QLcol==3)&&(altkey))    UsbKeyboard.sendKeyStroke(key_to_send,MOD_ALT_LEFT);  
      if((QLrow==7)&&(QLcol==5)&&(altkey))    UsbKeyboard.sendKeyStroke(key_to_send,MOD_ALT_LEFT);  
      if((QLrow==7)&&(QLcol==7)&&(altkey))    UsbKeyboard.sendKeyStroke(KEY_HOME);  
      if((QLrow==7)&&(QLcol==8)&&(altkey))    UsbKeyboard.sendKeyStroke(KEY_END);  
      if((QLrow==7)&&(QLcol==5)&&(altkey)&&(ctrlkey)) UsbKeyboard.sendKeyStroke(KEY_DELETE,5); // This is control-alt-delete; sends a double-bitshifted '1'
      if((QLrow==0)&&(QLcol==3)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F6);
      if((QLrow==0)&&(QLcol==4)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F7);
      if((QLrow==0)&&(QLcol==5)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F8);
      if((QLrow==0)&&(QLcol==6)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F9);
      if((QLrow==0)&&(QLcol==10)&&(shiftkey)) UsbKeyboard.sendKeyStroke(KEY_F10);
    }
    else {
      if((!capsstate)&&(!ctrlkey)&&(!shiftkey)&&(!altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow]);
      if ((capsstate)&&(!ctrlkey)&&(!shiftkey)&&(!altkey)&&((key_to_send<KEY_A)||(key_to_send>KEY_Z)))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow]);
      if(capsstate&&(!ctrlkey)&&(!shiftkey)&&(!altkey)&&((key_to_send>=KEY_A)&&(key_to_send<=KEY_Z)))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_SHIFT_LEFT);
      if((ctrlkey)&&(!shiftkey)&&(!altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_CONTROL_LEFT);
      if((!ctrlkey)&&(shiftkey)&&(!altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_SHIFT_LEFT);
      if((!ctrlkey)&&(!shiftkey)&&(altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_ALT_LEFT);
    }
  }

  ///////////////////////////////////////////////////////////
  // When key remains pressed, do nothing for a little while
  ///////////////////////////////////////////////////////////
  if((QLcol == QLcol_old)&&(QLrow==QLrow_old)&&(QLcol!=0)&&(sentonce==keys[QLcol-1][QLrow])&&(tt<200)) {
    //Serial.println(keys[QLcol-1][QLrow]);
    tt++;
  }

  //////////////////////////////////////
  // "Repeat" mode after a little while
  //////////////////////////////////////
  if((QLcol == QLcol_old)&&(QLrow==QLrow_old)&&(QLcol!=0)&&(sentonce==keys[QLcol-1][QLrow])&&(tt>=200)) {
    //Serial.println(keys[QLcol-1][QLrow]);
    tt++;
    delayMs(50);
    // Exceptions; especially assigned key combinations (Backspace, delete, home, end, pgup, pgdown, F6-F10)
    if(((QLrow==7)&&((QLcol==3)||(QLcol==5)||(QLcol==7)||(QLcol==8))&&((altkey)||(ctrlkey)))||(((QLcol==3)||(QLcol==4)||(QLcol==5)||(QLcol==9)||(QLcol==10))&&(QLrow==0)&&(shiftkey))) {
      if((QLrow==7)&&(QLcol==3)&&(ctrlkey))   UsbKeyboard.sendKeyStroke(KEY_BACKSPACE);  
      if((QLrow==7)&&(QLcol==5)&&(ctrlkey)&&(!altkey))   UsbKeyboard.sendKeyStroke(KEY_DELETE);  
      if((QLrow==7)&&(QLcol==7)&&(ctrlkey))   UsbKeyboard.sendKeyStroke(KEY_PGUP);  
      if((QLrow==7)&&(QLcol==8)&&(ctrlkey))   UsbKeyboard.sendKeyStroke(KEY_PGDOWN);  
      if((QLrow==7)&&(QLcol==3)&&(altkey))    UsbKeyboard.sendKeyStroke(key_to_send,MOD_ALT_LEFT);  
      if((QLrow==7)&&(QLcol==5)&&(altkey))    UsbKeyboard.sendKeyStroke(key_to_send,MOD_ALT_LEFT);  
      if((QLrow==7)&&(QLcol==7)&&(altkey))    UsbKeyboard.sendKeyStroke(KEY_HOME);  
      if((QLrow==7)&&(QLcol==8)&&(altkey))    UsbKeyboard.sendKeyStroke(KEY_END);  
      if((QLrow==7)&&(QLcol==5)&&(altkey)&&(ctrlkey)) UsbKeyboard.sendKeyStroke(KEY_DELETE,5); // This is control-alt-delete; sends a double-bitshifted '1'
      if((QLrow==0)&&(QLcol==3)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F6);
      if((QLrow==0)&&(QLcol==4)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F7);
      if((QLrow==0)&&(QLcol==5)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F8);
      if((QLrow==0)&&(QLcol==6)&&(shiftkey))  UsbKeyboard.sendKeyStroke(KEY_F9);
      if((QLrow==0)&&(QLcol==10)&&(shiftkey)) UsbKeyboard.sendKeyStroke(KEY_F10);
   }
    else {
      if((!capsstate)&&(!ctrlkey)&&(!shiftkey)&&(!altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow]);
      if ((capsstate)&&(!ctrlkey)&&(!shiftkey)&&(!altkey)&&((key_to_send<KEY_A)||(key_to_send>KEY_Z)))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow]);
      if(capsstate&&(!ctrlkey)&&(!shiftkey)&&(!altkey)&&((key_to_send>=KEY_A)&&(key_to_send<=KEY_Z)))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_SHIFT_LEFT);
      if((ctrlkey)&&(!shiftkey)&&(!altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_CONTROL_LEFT);
      if((!ctrlkey)&&(shiftkey)&&(!altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_SHIFT_LEFT);
      if((!ctrlkey)&&(!shiftkey)&&(altkey))
        UsbKeyboard.sendKeyStroke(keys[QLcol-1][QLrow],MOD_ALT_LEFT);
    }
  }

  ///////////////////////////////////////////////////
  // Reset counters when nothing seems to be pressed
  ///////////////////////////////////////////////////
  if(QLcol==0){
    tt = 0;
    sentonce = 0;
  }
  
  //////////////////////////
  // Prepare for next cycle
  //////////////////////////
  QLcol_old = QLcol;
  QLcol = 0;
  QLrow_old = QLrow;
  QLrow = 0;
//  delayMs(50);
}

